(*******************************************************************************
 *                SaveGame config.dat fixer for faulty settings                *
 *      when using Glide (GlideWrapper) in Need For Speed High Stakes (IV)     *
 *                                                                             *
 *                                                                             *
 * Copyright (C) 2008 Mihai Moldovan <ionic@ionic.de>                          *
 *                                                                             *
 * This program is free software; you can redistribute it and/or modify it     *
 * under the terms of the GNU General Public License as published by the Free  *
 * Software Foundation; either version 2 of the License, or (at your option)   *
 * any later version.                                                          *
 *                                                                             *
 * This program is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       *
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    *
 * more details.                                                               *
 *                                                                             *
 * You should have received a copy of the GNU General Public License along     *
 * with this program; if not, write to the Free Software Foundation, Inc.,     *
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.                 *
 *******************************************************************************)


unit UGlideFix;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls;

type

  { TTMainForm }

  TTMainForm = class(TForm)
    Button_Patch: TButton;
    Button_Open: TButton;
    Edit_Input_File: TEdit;
    Label_Input_File: TLabel;
    OpenDialog: TOpenDialog;
    procedure Button_OpenClick(Sender: TObject);
    procedure Button_PatchClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  TMainForm: TTMainForm;

implementation

{ TTMainForm }

procedure TTMainForm.Button_OpenClick(Sender: TObject);
begin
  OpenDialog.InitialDir := GetCurrentDir ();
  if (OpenDialog.Execute = TRUE) then
    begin
      Edit_Input_File.Text := OpenDialog.FileName;

      with Edit_Input_File.Font do
        begin
          Color := clGreen;
          Style := [ fsBold ];
      end;

      ShowMessage ('OK! You can now apply the patch.');
  end;

end;

procedure TTMainForm.Button_PatchClick(Sender: TObject);
var
  FileName, BFileName: String;
  FHandle: TFileStream;
  Seek: Cardinal;
begin
  if (OpenDialog.FileName = '') then
    begin
      if (MessageDlg ('Continue?', 'You have not selected the file to patch. Are you sure that you want to proceed? If the default file already matches your situation, it is save to do so. Otherwise, please select "No" and select a file.', mtCustom, [mbYes, mbNo], 0, mbYes) = mrYes) then
        FileName := GetCurrentDir () + '/' + Edit_Input_File.Text;
    end
  else
    FileName := OpenDialog.FileName;

  // Okidekey, so we now do indeed have the file.
  // ... wait. Is it really there? Test for it.
  if (FileExists (FileName) = TRUE) then
    begin
      // Wohoo, it works!
      // ... wait. Is it possible to read and write to it as well? Test for this.
      if (((FileGetAttr (FileName) AND faReadOnly) <> faReadOnly) AND (FileIsReadable (FileName) = TRUE) AND (FileIsWritAble (FileName) = TRUE)) then
        begin
          // Very cool.
          // So first, create a backup.
          // Construct the backup file name (we will simply add '.bak' to the old name.)
          BFileName := ExtractFilePath (FileName) + ChangeFileExt (ExtractFileName (FileName), '.dat.bak');
          if (CopyFile (PChar (FileName), PChar (BFileName), FALSE) = FALSE) then
            Raise Exception.Create ('I could not create a backup of the original file! Please check, if the backup file (' + BFileName + ') is still in use by some program, or if the permissions are correct.')
          else
            begin
              try
                FHandle := TFileStream.Create (FileName, fmOpenReadWrite);
                if (FHandle.Size > 337) then
                  begin
                    Seek := FHandle.Seek (336, soFromBeginning);
                    FHandle.WriteByte ($00);
                  end
                else
                  ShowMessage ('Sorry, the requested file is too small. Maybe it is corrupted or actually no config.dat?');
              finally
                ShowMessage ('Your file has been successfully patched! Happy playing and good luck!');
                FHandle.Free;
              end;
          end;
        end
      else
        ShowMessage ('Sorry, but the file is not read- or writable.');
    end
  else
    ShowMessage ('Sorry, the requested file does not exist.');

end;

initialization
  {$I UGlideFix.lrs}

end.

